/*!
 * \brief Controller for the step that tries to establish a PACE
 *        connection using the card's Pin.
 *
 * \copyright Copyright (c) 2016-2020 Governikus GmbH & Co. KG, Germany
 */

#pragma once

#include "AbstractState.h"
#include "context/WorkflowContext.h"
#include "GenericContextContainer.h"

class test_StateEstablishPaceChannel;

namespace governikus
{

class StateEstablishPaceChannel
	: public AbstractState
	, public GenericContextContainer<WorkflowContext>
{
	Q_OBJECT
	friend class StateBuilder;
	friend class ::test_StateEstablishPaceChannel;

	private:
		explicit StateEstablishPaceChannel(const QSharedPointer<WorkflowContext>& pContext);

		PacePasswordId mPasswordId;

		virtual void run() override;
		virtual void onUserCancelled() override;

		void abort();
		void handleNpaPosition(CardReturnCode pReturnCode);

	private Q_SLOTS:
		void onEstablishConnectionDone(QSharedPointer<BaseCardCommand> pCommand);

	Q_SIGNALS:
		void firePaceChannelEstablished();
		void firePaceChannelInoperative();
		void fireAbortAndUnfortunateCardPosition();
};

} // namespace governikus
